local aucmd_dict = {
    FileType = {
	-- Set tabstop to 2 for Dart, Vue, JavaScript, TypeScript, and JSON files
        {
            pattern = "dart,vue,javascript,typescript,json",
            callback = function()
                vim.opt_local.tabstop = 2
                vim.opt_local.softtabstop = 2
                vim.opt_local.shiftwidth = 2
            end,
        },
    },

    BufWritePre = {
	-- Remove trailing whitespace on save
        {
            command = [[%s/\s\+$//e]],
        },

	-- Run goimports on save
        {
            pattern = "*.go",
            command = 'silent! lua require(\'go.format\').goimport()'
        },
    },

    BufRead = {
	-- Return cursor to where it was previously when re-opening a file
        {
            command = [[if @% !~# '\.git[\/\\]COMMIT_EDITMSG$' && line("'\"") > 1 && line("'\"") <= line("$") | exe "normal! g`\"" | endif]]
        },

	-- Set syntax for Dockerfiles
        {
            pattern = { '*.docker' },
            callback = function()
                vim.opt_local.syntax = 'dockerfile'
            end
        }
    },

    BufNewFile = {
	-- Set syntax for Dockerfiles
        {
            pattern = { '*.docker' },
            callback = function()
                vim.opt_local.syntax = 'dockerfile'
            end
        }
    },

    VimLeave = {
	-- Save session on exit
        {
            command = [[mksession! ~/.cache/nvim/session/shutdown_session.vim]]
        },
    },
}

for event, opt_tbls in pairs(aucmd_dict) do
    for _, opt_tbl in pairs(opt_tbls) do
        vim.api.nvim_create_autocmd(event, opt_tbl)
    end
end

